@@ -1,6 +1,7 @@
+from copy import deepcopy
 from anasymod.templates.templ import JinjaTempl
 from anasymod.generators.gen_api import SVAPI, ModuleInst
-from anasymod.sim_ctrl.ctrlifc_datatypes import DigitalSignal
+from anasymod.sim_ctrl.datatypes import DigitalSignal
 
 
 class ModuleRegMapSimCtrl(JinjaTempl):
@@ -20,26 +21,20 @@ class ModuleRegMapSimCtrl(JinjaTempl):
         clk = DigitalSignal(name='clk', width=1, abspath=None)
         module.add_input(io_obj=clk)
 
-        # Add reset
-        rst = DigitalSignal(name='rst', width=1, abspath=None)
-        module.add_input(io_obj=rst)
-
-        # Add I/Os to CPU subsystem
-        i_addr = DigitalSignal(name='i_addr', width=8, abspath=None)
-        module.add_input(io_obj=i_addr)
-        i_data = DigitalSignal(name='i_data', width=32, abspath=None)
-        module.add_input(io_obj=i_data)
-        o_addr = DigitalSignal(name='o_addr', width=8, abspath=None)
-        module.add_input(io_obj=o_addr)
+        o_ctrl = DigitalSignal(name='o_ctrl', width=32, abspath=None)
+        module.add_input(io_obj=o_ctrl)
         o_data = DigitalSignal(name='o_data', width=32, abspath=None)
         module.add_output(io_obj=o_data)
+        i_ctrl = DigitalSignal(name='i_ctrl', width=32, abspath=None)
+        module.add_input(io_obj=i_ctrl)
+        i_data = DigitalSignal(name='i_data', width=32, abspath=None)
+        module.add_input(io_obj=i_data)
 
         # Add I/Os to Design (probes and Conrol Parameters)
         module.add_outputs(io_objs=crtl_inputs, connections=crtl_inputs)
         module.add_inputs(io_objs=ctrl_outputs, connections=ctrl_outputs)
 
         module.generate_header()
-        #ToDo: currently clk will be treated as inputs to this wrapper, might be driven from PS at one point
 
         #####################################################
         # Initialize Default Values for ControlInfrastructure Parameters
@@ -48,7 +43,7 @@ class ModuleRegMapSimCtrl(JinjaTempl):
         self.init_ctrlios = SVAPI()
 
         for parameter in crtl_inputs:
-            default_signal = parameter
+            default_signal = deepcopy(parameter)
             default_signal.name = str(default_signal.name) + '_def'
             self.init_ctrlios.gen_signal(io_obj=default_signal)
             self.init_ctrlios.assign_to(io_obj=default_signal, exp=default_signal.init_value)
@@ -62,7 +57,8 @@ class ModuleRegMapSimCtrl(JinjaTempl):
 
         self.probes_combomux_cases.indent(quantity=3)
         for probe in ctrl_outputs:
-            self.probes_combomux_cases.writeln(f"'d{probe.o_addr}: o_data = {probe.name};")
+            if probe.o_addr is not None:
+                self.probes_combomux_cases.writeln(f"'d{probe.o_addr}: o_data_reg = {probe.name};")
 
         #####################################################
         # Combo mux for Probes section
@@ -72,16 +68,25 @@ class ModuleRegMapSimCtrl(JinjaTempl):
         self.params_regmap = SVAPI()
 
         for param in crtl_inputs:
-            self.params_regmap.writeln(f"always @(posedge clk) begin")
-            self.params_regmap.writeln(f"    if (rst == 'b1) begin")
-            self.params_regmap.writeln(f"        {param.name} <= {param.name}_def; // use VIO defaults")
-            self.params_regmap.writeln(f"    end else if (i_addr == 'd{param.i_addr}) begin ")
-            self.params_regmap.writeln(f"        {param.name} <= i_data;")
-            self.params_regmap.writeln(f"    end else begin")
-            self.params_regmap.writeln(f"        {param.name} <= {param.name};")
-            self.params_regmap.writeln(f"    end")
-            self.params_regmap.writeln(f"end")
-            self.params_regmap.writeln(f"")
+            # create a reg signal
+            reg_signal = deepcopy(param)
+            reg_signal.name = f'{param.name}_reg'
+            self.params_regmap.gen_signal(reg_signal)
+
+            # assign to the reg signal
+            self.params_regmap.writeln(f'assign {param.name} = {param.name}_reg;')
+
+            # update the reg signal
+            self.params_regmap.writeln(f'''\
+always @(posedge clk) begin
+    if (i_rst == 'b1) begin
+        {param.name}_reg <= {param.name}_def; // use VIO defaults
+    end else if ((i_valid == 1'b1) && (i_addr == 'd{param.i_addr})) begin
+        {param.name}_reg <= i_data;
+    end else begin
+        {param.name}_reg <= {param.name}_reg;
+    end
+end''')
 
     TEMPLATE_TEXT = '''
 `timescale 1ns/1ps
@@ -90,16 +95,30 @@ class ModuleRegMapSimCtrl(JinjaTempl):
 
 {{subst.module_ifc.text}}
 
+    // break out signals in o_ctrl    
+    logic [7:0] o_addr;
+    assign o_addr = o_ctrl[7:0];
+    
+    // break out signals in i_ctrl
+    logic i_rst;
+    assign i_rst = i_ctrl[31];
+    
+    logic i_valid;
+    assign i_valid = i_ctrl[30];
+    
+    logic [7:0] i_addr;
+    assign i_addr = i_ctrl[7:0];
+    
     // Initial values for parameters
-    {{subst.init_ctrlios.text}}  
-    wire o_data_def;
-    assign o_data_def = 0;
+    {{subst.init_ctrlios.text}}
 
 	// combo mux for reading outputs from design
+	logic [31:0] o_data_reg;
+	assign o_data = o_data_reg;
 	always @* begin
 		case (o_addr)
 {{subst.probes_combomux_cases.text}}
-		    default: o_data = o_data_def;
+		    default: o_data_reg = 0;
 		endcase
 	end
 
@@ -108,12 +127,4 @@ class ModuleRegMapSimCtrl(JinjaTempl):
 
 endmodule
 `default_nettype wire
-'''
-
-def main():
-    from anasymod.structures.structure_config import StructureConfig
-    from anasymod.config import EmuConfig
-    print(ModuleRegMapSimCtrl(scfg=StructureConfig(prj_cfg=EmuConfig(root='test', cfg_file=''))).render())
-
-if __name__ == "__main__":
-    main()
\ No newline at end of file
+'''
\ No newline at end of file
